import { METADATA } from "./constants";
import { DataType } from "./DataType";
import { Errors } from "../system/Errors";
const getReflectedDataType = (prototype, propertyKey) => {
    let t = Reflect.getMetadata("design:type", prototype, propertyKey);
    if (!t) {
        throw Errors.SYSTEM_ERROR(`Cannot get reflected data type for ${prototype.constructor.name}:${propertyKey}`);
    }
    switch (t.name) {
        case "GeoPoint":
            return DataType.POINT;
        case "Boolean":
            return DataType.BOOLEAN;
        case "Number":
            return DataType.NUMBER;
        case "String":
            return DataType.STRING;
        case "Array":
            return DataType.ARRAY;
        default:
            return DataType.OBJECT;
    }
};
export const initObjectMetadata = (prototype) => {
    if (prototype === Object.prototype) {
        return {};
    }
    if (!prototype.hasOwnProperty(METADATA)) {
        const superPrototype = initObjectMetadata(Object.getPrototypeOf(prototype));
        prototype[METADATA] = {
            ...superPrototype,
            id: getTableName(prototype.constructor),
            fields: superPrototype.fields ? superPrototype.fields.map((f) => ({ ...f })) : [],
        };
    }
    return prototype[METADATA];
};
export const initFieldMetadata = (prototype, propertyKey) => {
    let meta = initObjectMetadata(prototype);
    let fieldIndex = meta.fields.findIndex((f) => f.name === propertyKey);
    if (fieldIndex < 0) {
        const field = {
            name: propertyKey,
            dataType: getReflectedDataType(prototype, propertyKey),
            description: "",
        };
        meta.fields.push(field);
        return field;
    }
    else {
        //-- shadow clone the field
        const field = { ...meta.fields[fieldIndex] };
        meta.fields[fieldIndex] = field;
        return field;
    }
};
export const getObjectMetadata = (obj) => obj.prototype[METADATA];
export const setObjectMetadata = (obj, metadata) => {
    obj.prototype[METADATA] = metadata;
};
export const getFieldMetadata = (cls, key) => {
    const clsMeta = getObjectMetadata(cls);
    return clsMeta?.fields.find((f) => f.name === key);
};
export const getTableName = (model) => model?.name.toLowerCase();
