import { SerializedValueProvider } from "./request-serializer";
export class CachedDataContainer {
    valueGetterFn;
    valueSetterFn;
    dataRetentionDurationMs;
    cache = {};
    valueProvider;
    constructor(valueGetterFn, valueSetterFn, dataRetentionDurationMs = 30000) {
        this.valueGetterFn = valueGetterFn;
        this.valueSetterFn = valueSetterFn;
        this.dataRetentionDurationMs = dataRetentionDurationMs;
        this.valueProvider = new SerializedValueProvider(this.valueGetterFn);
    }
    async getValue(...args) {
        const key = args.map((arg) => String(arg)).join(":");
        let cache = this.cache[key];
        if (cache === undefined) {
            cache = {
                lastFetchTimestamp: 0,
            };
            this.cache[key] = cache;
        }
        if (cache.lastFetchTimestamp + this.dataRetentionDurationMs < Date.now()) {
            const value = await this.valueProvider.getValue(...args);
            cache.value = value;
            cache.lastFetchTimestamp = Date.now();
        }
        return cache.value;
    }
    async setValue(value, ...args) {
        const key = args.map((arg) => String(arg)).join(":");
        let cache = this.cache[key];
        if (cache === undefined) {
            cache = {
                lastFetchTimestamp: Date.now(),
            };
            this.cache[key] = cache;
        }
        cache.value = value;
        await this.valueSetterFn?.(value, ...args);
    }
}
