export class SerializedValueProvider {
    fn;
    constructor(fn) {
        this.fn = fn;
    }
    resolvers = {};
    async getValue(...args) {
        const key = args.map((arg) => String(arg)).join(":");
        let resolvers = this.resolvers[key];
        if (!resolvers) {
            resolvers = {
                fetching: false,
                resolvers: [],
            };
            this.resolvers[key] = resolvers;
        }
        if (resolvers.fetching) {
            //-- let's queue here
            return new Promise((resolve) => {
                resolvers.resolvers.push(resolve);
            });
        }
        //-- not fetching, let's do it
        resolvers.fetching = true;
        const value = await this.fn(...args);
        //-- notify those waiting
        for (const resolveFn of resolvers.resolvers) {
            resolveFn(value);
        }
        resolvers.fetching = false;
        resolvers.resolvers = [];
        return value;
    }
}
