import { initObjectMetadata } from "../common/metadata";
import { getServiceProvider } from "./injection";
import { ErrorBuilder } from "../system/Errors";
export const Register = (config) => (target) => {
    const serviceProvider = getServiceProvider();
    serviceProvider.register(target, config?.mode || "singleton");
};
export const Provider = (baseClass, mode) => (constructor) => {
    Register({ mode: "singleton" })(constructor);
    const metadata = initObjectMetadata(constructor.prototype);
    metadata.injectionMode = mode || "singleton";
    const serviceProvider = getServiceProvider();
    serviceProvider.addProvider(baseClass, constructor);
};
export const Inject = (baseClass, required = true) => ((target, propertyKey, index) => {
    if (propertyKey !== undefined) {
        throw ErrorBuilder.forInvalidInjection(`@Inject decorator used outside constructor method ${target.name}:${propertyKey}:${index}`);
    }
    let metadata = initObjectMetadata(target.prototype);
    if (!metadata.constructorDependencies) {
        metadata.constructorDependencies = (Reflect.getMetadata("design:paramtypes", target) || []).map((ctor) => ({ ctor, required: true }));
    }
    if (metadata.constructorDependencies) {
        metadata.constructorDependencies[index].ctor = baseClass;
        metadata.constructorDependencies[index].required = required;
    }
});
export const Injectable = () => (_target) => {
    //-- this is only being used to trigger parameter metadata in constructor
};
