import { ObjectMetadata } from "../common/ObjectMetadata";
import { Mime } from "../common/Mime";
import { ObjectFieldMetadata } from "../common/ObjectFieldMetadata";
import { DataType } from "../common/DataType";
import { PrimitiveArray } from "../common/types";
import { CascadeType } from "../common/CascadeType";
import { HttpMethod } from "../http/HttpMethod";
export interface VectorProps {
    /**
     * The super set of the provided value.
     */
    superSet?: PrimitiveArray;
    /**
     * Allow the provided vector to contain duplicated values. Default to true if omitted.
     */
    allowDuplicated?: boolean;
    /**
     * Min length of the vector. Default to be zero.
     */
    minLength?: number;
    /**
     * Max length of the vector. Default to be unlimited.
     */
    maxLength?: number;
    /**
     * Data type of the vector element.
     */
    elementDataType?: DataType;
    /**
     * Allow null value in the array.
     */
    allowNull?: boolean;
}
export interface RangeProps {
    /**
     * Min value of the range.
     */
    min?: number;
    /**
     * Max value of the range.
     */
    max?: number;
    /**
     * If true, include min value in the range.
     */
    minExclusive?: boolean;
    /**
     * If true, include max value in the range.
     */
    maxExclusive?: boolean;
}
export interface MimeProps {
    type?: Mime;
    maxFileKB?: number;
    public?: boolean;
    keepOnRemove?: boolean;
}
export interface DtoFieldMetadata extends ObjectFieldMetadata {
    /**
     * Whether the field is multi-localization
     */
    isMultiLocale?: boolean;
    /**
     * Name of the column this field support for localization
     */
    multiLocaleColumn?: string;
    /**
     * Whether this field is primary key.
     */
    pk?: {
        /**
         * Auto-incrementing PK, default to true if omitted.
         */
        isAutoIncrement?: boolean;
    };
    fk?: {
        /**
         * Id of the model to refer as foreign key.
         */
        dtoId?: string;
        /**
         * Whether to cascade on delete.
         */
        cascade?: CascadeType;
    };
    /**
     * Default value for the field if not provided
     */
    defaultValue?: any;
    /**
     * This field value is symbol and must be treated at whole in search
     */
    isSymbol?: boolean;
    /**
     * Regex express check
     */
    regex?: string;
    /**
     * Accept null as value.
     */
    allowNull?: boolean;
    /**
     * Floating numeric value.
     */
    floating?: boolean;
    /**
     * Value in range. Inclusive default to true if omitted.
     */
    rangeProps?: RangeProps;
    /**
     * Value belong to this enum.
     */
    enum?: PrimitiveArray;
    /**
     * The provided value is an array.
     */
    vectorProps?: VectorProps;
    /**
     * Use to validate in case DateType is Object, or in case DataType is Array and vectorProps.elementDataType is Object.
     */
    elementDto?: DtoMetadata;
    /**
     * This value is generated from server and must not be supplied from client.
     */
    serverValue?: boolean;
    /**
     * Mime type for better client-side rendering.
     */
    mimeProps?: MimeProps;
    /**
     * Whether current field is timestamp
     */
    isTimestamp?: boolean;
    /**
     * Whether to use TEXT to store the string value at provided length.
     */
    textLength?: number;
    /**
     * Whether to store this as 64-bit big integer.
     */
    isBigInt?: boolean;
    /**
     * Whether this dto has many other dto as child element
     */
    hasMany?: {
        /** id of the model in this relation ship */
        relationDto: DtoMetadata;
        /** the field in the target model that is FK to this model */
        column: string;
        /**
         * 1 - 1 relationship
         */
        single?: boolean;
    };
}
export interface DtoMetadata extends ObjectMetadata {
    /**
     * Ignoring CRUD methods if using with CrudHttpProvider
     */
    ignoreCrud?: HttpMethod[];
    relaxed?: boolean;
    description?: string;
    primitiveType?: DataType;
    fields: DtoFieldMetadata[];
}
