var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { DataType } from "../common/DataType";
import { getObjectMetadata, setObjectMetadata } from "../common/metadata";
import { Data } from "../dto/decorator";
import { DeepPartialClass, partialize } from "../dto/deep-partial";
import { GetManyQueries, GetManyResponseBody } from "./request-response";
import { ElementDto } from "../dto/decorator";
import { RangeQueryDto } from "./RangeQueryDto";
export const getCreateManyBodyValidator = (modelMetadata) => {
    //-- body ------------------------------------
    const bodyDto = { id: "", fields: [] };
    const fieldMetadata = {};
    fieldMetadata.name = "records";
    fieldMetadata.description = "Array of records to be created.";
    fieldMetadata.dataType = DataType.ARRAY;
    fieldMetadata.isRequired = true;
    fieldMetadata.vectorProps = {
        elementDataType: DataType.OBJECT,
        minLength: 0,
    };
    fieldMetadata.elementDto = getBodyDtoRecordClass(modelMetadata, "create_body");
    bodyDto.fields.push(fieldMetadata);
    return bodyDto;
};
export const getCreateManyResponseValidator = (modelMetadata) => {
    const bodyDto = { id: "", fields: [] };
    //-- records
    const recordsFieldMetadata = {};
    recordsFieldMetadata.name = "records";
    recordsFieldMetadata.description = "Array of records had just been created.";
    recordsFieldMetadata.dataType = DataType.ARRAY;
    recordsFieldMetadata.isRequired = true;
    recordsFieldMetadata.vectorProps = {
        elementDataType: DataType.OBJECT,
        minLength: 0,
    };
    recordsFieldMetadata.elementDto = getBodyDtoRecordClass(modelMetadata, "create_response");
    bodyDto.fields.push(recordsFieldMetadata);
    return bodyDto;
};
export const getGetManyResponseValidator = (modelMetadata) => {
    const bodyDto = { id: "", fields: [] };
    //-- total
    const totalFieldMetadata = {};
    totalFieldMetadata.name = "total";
    totalFieldMetadata.description = "Number of records found, zero is returned if the request is not paged.";
    totalFieldMetadata.dataType = DataType.NUMBER;
    totalFieldMetadata.isRequired = true;
    bodyDto.fields.push(totalFieldMetadata);
    //-- records
    const recordsFieldMetadata = {};
    recordsFieldMetadata.name = "records";
    recordsFieldMetadata.description = "The found records. All records will be returned if the request is not paged.";
    recordsFieldMetadata.dataType = DataType.ARRAY;
    recordsFieldMetadata.isRequired = true;
    recordsFieldMetadata.vectorProps = {
        elementDataType: DataType.OBJECT,
        minLength: 0,
    };
    recordsFieldMetadata.elementDto = getBodyDtoRecordClass(modelMetadata, "get_response");
    bodyDto.fields.push(recordsFieldMetadata);
    return bodyDto;
};
export const getBodyDtoRecordClass = (modelMetadata, operation, parentModelMetadata) => {
    const instanceDtoMetadata = { id: "", fields: [] };
    let baseFields = modelMetadata.fields.filter((f) => !f.hasMany && !f.multiLocaleColumn);
    if (operation === "create_response") {
        baseFields = baseFields.filter((field) => field.pk ||
            field.serverValue ||
            field.mimeProps ||
            (!field.isRequired && field.defaultValue !== undefined));
    }
    if (operation === "create_body" || operation === "update_body") {
        baseFields = baseFields.filter((field) => 
        //-- a normal field
        (!field.serverValue && !field.pk?.isAutoIncrement) ||
            //-- model repository has already applied parent id to this field, so exclude it here
            (!!parentModelMetadata && !parentModelMetadata.fields.find((f) => f.hasMany?.column === field.name)));
    }
    const localeFields = modelMetadata.fields.filter((field) => field.multiLocaleColumn);
    const hasManyFields = modelMetadata.fields.filter((field) => !!field.hasMany);
    instanceDtoMetadata.fields.push(...baseFields.map((field) => ({
        ...field,
        elementDto: field.elementDto && (operation === "update_body" ? partialize(field.elementDto) : field.elementDto),
        isRequired: operation === "query" ||
            operation === "update_body" ||
            (operation === "get_response" && !parentModelMetadata)
            ? false
            : field.isRequired && !field.isMultiLocale,
    })), ...hasManyFields.map((field) => {
        const elementDto = getBodyDtoRecordClass(field.hasMany.relationDto, operation, modelMetadata);
        return {
            description: `All records of "${field.hasMany?.relationDto.id}" table in this relationship.`,
            name: field.name,
            dataType: field.hasMany?.single ? DataType.OBJECT : DataType.ARRAY,
            isRequired: !field.hasMany?.single && operation === "create_response",
            vectorProps: field.hasMany?.single
                ? undefined
                : {
                    elementDataType: DataType.OBJECT,
                    minLength: 0,
                },
            elementDto: {
                ...elementDto,
                fields: 
                //-- if update / create then the pk column in has many relationship is automatically resolved
                (operation === "create_body" || operation === "update_body"
                    ? elementDto.fields.filter((f) => f.name !== field.hasMany?.column)
                    : elementDto.fields).map((f) => (operation === "update_body" ? { ...f, isRequired: false, allowNull: true } : f)),
            },
        };
    }), ...localeFields.map((field) => {
        return {
            description: "Translation for " + field.multiLocaleColumn,
            name: field.name,
            isRequired: false,
            dataType: DataType.OBJECT,
        };
    }));
    return instanceDtoMetadata;
};
export const projectionFieldDto = (modelMetadata) => {
    const dto = {};
    dto.name = "projection";
    dto.description = "Project the returning records to contain only certain fields. Omit to return all.";
    dto.dataType = DataType.ARRAY;
    dto.vectorProps = {
        superSet: modelMetadata.fields.map((field) => field.name),
        allowDuplicated: false,
        minLength: 1,
        elementDataType: DataType.STRING,
    };
    return dto;
};
export const getRequestQueryFieldFromModel = (modelMetadata) => {
    return modelMetadata.fields.map((modelFieldMetadata) => {
        const queryFieldMedata = {};
        queryFieldMedata.name = modelFieldMetadata.name;
        queryFieldMedata.description = modelFieldMetadata.description;
        queryFieldMedata.isRequired = false;
        if (modelFieldMetadata.pk || modelFieldMetadata.fk || modelFieldMetadata.isSymbol) {
            //-- if is PK or FK then allow to search by array of ids
            queryFieldMedata.dataType = DataType.ARRAY;
            queryFieldMedata.vectorProps = {
                elementDataType: modelFieldMetadata.dataType,
                allowDuplicated: false,
                allowNull: modelFieldMetadata.allowNull,
                minLength: 1,
            };
        }
        else if (modelFieldMetadata.hasMany) {
            const dto = {
                id: "",
                fields: getRequestQueryFieldFromModel(modelFieldMetadata.hasMany.relationDto).filter((f) => f.name !== modelFieldMetadata.hasMany?.column),
            };
            dto.relaxed = true;
            queryFieldMedata.dataType = DataType.OBJECT;
            queryFieldMedata.elementDto = dto;
        }
        else {
            if (modelFieldMetadata.enum) {
                queryFieldMedata.dataType = DataType.ARRAY;
                queryFieldMedata.vectorProps = {
                    superSet: modelFieldMetadata.enum,
                    allowDuplicated: false,
                    minLength: 1,
                    elementDataType: modelFieldMetadata.enum.filter((v) => typeof v === "number").length
                        ? DataType.NUMBER
                        : DataType.STRING,
                    allowNull: modelFieldMetadata.allowNull,
                };
            }
            else {
                switch (modelFieldMetadata.dataType) {
                    case DataType.STRING:
                        // to be processed as regex in database adapter
                        queryFieldMedata.dataType = DataType.STRING;
                        break;
                    case DataType.NUMBER:
                        queryFieldMedata.dataType = DataType.OBJECT;
                        queryFieldMedata.elementDto = getObjectMetadata(RangeQueryDto);
                        break;
                    case DataType.BOOLEAN:
                        queryFieldMedata.dataType = DataType.BOOLEAN;
                        queryFieldMedata.allowNull = modelFieldMetadata.allowNull;
                        break;
                }
            }
        }
        return queryFieldMedata;
    });
};
export const queryFieldDto = (modelMetadata) => {
    const fieldsDtoMetadata = {
        id: "",
        fields: getRequestQueryFieldFromModel(modelMetadata),
    };
    const field = {};
    field.name = "fields";
    field.description = "Filter records by value of their fields.";
    field.dataType = DataType.OBJECT;
    field.elementDto = fieldsDtoMetadata;
    return field;
};
export const getGetManyQueryValidator = (modelMetadata) => {
    //-- fields query
    const queryDto = {
        id: "",
        relaxed: true,
        fields: [queryFieldDto(modelMetadata)],
    };
    //-- locale query
    const localeDto = {};
    localeDto.name = "locale";
    localeDto.description = "Language code for localization";
    localeDto.isRequired = false;
    localeDto.dataType = DataType.STRING;
    queryDto.fields.push(localeDto);
    //-- search query ---------------------------------------------------
    const searchDto = {};
    searchDto.name = "search";
    searchDto.description = "Filter records for their @Searchable denoted fields.";
    searchDto.dataType = DataType.STRING;
    queryDto.fields.push(searchDto);
    //-- projection query ---------------------------------------------------
    queryDto.fields.push(projectionFieldDto(modelMetadata));
    //-- order query ---------------------------------------------------
    const sortDto = {};
    sortDto.name = "order";
    sortDto.description =
        "Sort the dataset before getting result by order given in the array. Field appears first in the array will be sorted first.";
    sortDto.dataType = DataType.ARRAY;
    sortDto.vectorProps = {
        elementDataType: DataType.OBJECT,
        minLength: 1,
    };
    //-- Dynamic value for decorator is not allowed, so we create
    //-- and inject metadata object to class to be retrieved later by validator.
    const sortElementDtoMetadata = { id: "", fields: [] };
    sortElementDtoMetadata.fields.push(...modelMetadata.fields
        .filter((f) => !f.hasMany)
        .map((field) => {
        const fieldMetadata = {};
        fieldMetadata.name = field.name;
        fieldMetadata.isRequired = false;
        fieldMetadata.dataType = DataType.STRING;
        fieldMetadata.enum = ["asc", "desc"];
        return fieldMetadata;
    }));
    sortDto.elementDto = sortElementDtoMetadata;
    queryDto.fields.push(sortDto);
    //-- limit query ---------------------------------------------------
    const limitDto = {};
    limitDto.name = "limit";
    limitDto.description = "Limit the number of returning result.";
    limitDto.dataType = DataType.NUMBER;
    limitDto.rangeProps = {
        min: 1,
    };
    queryDto.fields.push(limitDto);
    //-- page query ---------------------------------------------------
    const pageDto = {};
    pageDto.name = "page";
    pageDto.description = "Page the returning result, default to 1 if ommited. Has no effect if limit is not set.";
    pageDto.dataType = DataType.NUMBER;
    pageDto.rangeProps = {
        min: 1,
    };
    queryDto.fields.push(pageDto);
    return queryDto;
};
const getRetuningFieldMetadata = () => {
    const returningField = {};
    returningField.name = "returning";
    returningField.description = "Whether to return the affected records by this request.";
    returningField.dataType = DataType.BOOLEAN;
    returningField.isRequired = false;
    return returningField;
};
export const getUpdateManyQueryValidator = (modelMetadata) => {
    return {
        id: "",
        relaxed: true,
        fields: [getRetuningFieldMetadata(), queryFieldDto(modelMetadata)],
    };
};
export const getUpdateManyBodyValidator = (modelMetadata) => {
    const bodyDto = { id: "", fields: [] };
    const fieldMetadata = {};
    fieldMetadata.name = "update";
    fieldMetadata.isRequired = true;
    fieldMetadata.description = "The update data to apply to found records.";
    fieldMetadata.dataType = DataType.OBJECT;
    fieldMetadata.elementDto = getBodyDtoRecordClass(modelMetadata, "update_body");
    bodyDto.fields.push(fieldMetadata);
    return bodyDto;
};
export const getUpdateManyResponseValidator = (modelMetadata) => {
    const bodyDto = { id: "", fields: [] };
    const fieldMetadata = {};
    fieldMetadata.name = "modified";
    fieldMetadata.description = "Array of affected records.";
    fieldMetadata.dataType = DataType.ARRAY;
    fieldMetadata.vectorProps = { minLength: 0, elementDataType: DataType.OBJECT };
    fieldMetadata.elementDto = fieldMetadata.elementDto = getBodyDtoRecordClass(modelMetadata, "get_response");
    bodyDto.fields.push(fieldMetadata);
    return bodyDto;
};
export const getUpdateRecordsQueryValidator = () => {
    return {
        id: "",
        relaxed: true,
        fields: [getRetuningFieldMetadata()],
    };
};
export const getUpdateRecordsBodyValidator = (modelMetadata) => {
    const bodyDto = { id: "", fields: [] };
    const fieldMetadata = {};
    fieldMetadata.name = "records";
    fieldMetadata.isRequired = true;
    fieldMetadata.description = "The records with id and data to update";
    fieldMetadata.dataType = DataType.ARRAY;
    fieldMetadata.vectorProps = { minLength: 0, elementDataType: DataType.OBJECT };
    fieldMetadata.elementDto = getBodyDtoRecordClass(modelMetadata, "update_body");
    //-- each update record must contain id field
    fieldMetadata.elementDto.fields.push({
        name: "id",
        dataType: DataType.STRING,
        isRequired: true,
    });
    bodyDto.fields.push(fieldMetadata);
    return bodyDto;
};
export const GetManyResponseBodyClass = (cls) => {
    let _ = class _ extends GetManyResponseBody {
    };
    _ = __decorate([
        Data()
    ], _);
    const newMetadata = getObjectMetadata(_);
    const recordsMeta = newMetadata.fields.find((field) => field.name === "records");
    if (recordsMeta) {
        recordsMeta.elementDto = ElementDto(DeepPartialClass(cls)).elementDto;
    }
    return _;
};
export const GetManyQueriesClass = (cls) => {
    let _ = class _ extends GetManyQueries {
    };
    _ = __decorate([
        Data({ relaxed: true })
    ], _);
    setObjectMetadata(_, getGetManyQueryValidator(getObjectMetadata(cls)));
    return _;
};
