import { LogLevel } from "./LogLevel";
import { AbstractLogger } from "./AbstractLogger";
const getCircularReplacer = () => {
    const seen = new WeakSet();
    return (_, value) => {
        if (typeof value === "object" && value !== null) {
            if (seen.has(value)) {
                return;
            }
            seen.add(value);
        }
        return value;
    };
};
export class DefaultLogger extends AbstractLogger {
    logLimit;
    logMedia;
    constructor(logLimit, logMedia) {
        super(logLimit, logMedia);
        this.logLimit = logLimit;
        this.logMedia = logMedia;
    }
    serializeContent(logLevel, ...content) {
        let logContent = content.length <= 1 ? "" : "\n";
        for (let i = 0; i < content.length; i += 1) {
            logContent +=
                typeof content[i] === "string" ? content[i] : JSON.stringify(content[i], getCircularReplacer());
            if (i < content.length - 1) {
                logContent += "\n";
            }
        }
        return `[${LogLevel[logLevel]}][${this.getLogContext()}][${new Date().toISOString()}]: ` + logContent;
    }
}
