import { HttpMethod } from "../http/HttpMethod";
import { AbstractModel } from "./AbstractModel";
import { ModelFieldMetadata } from "./ModelFieldMetadata";
import { Constructor } from "../common/types";
import { OmitProps } from "../dto/decorator";
import { CascadeType } from "../common/CascadeType";
type ColumnProps = Omit<ModelFieldMetadata, OmitProps>;
export declare const Model: (config?: {
    /**
     * Description about this model.
     */
    description?: string;
    /**
     * Override a model by using it id so that any service that uses the old model will use this model instead.
     * To support backward compatible, this new model should extends from the old model.
     */
    override?: Constructor<AbstractModel>;
    /**
     * Ignore some http method to prevent auto generation of routes and/or endpoints
     */
    ignoreCrud?: HttpMethod[];
}) => <T extends Constructor<AbstractModel>>(constructor: T) => T;
export declare const Index: <T extends Record<string, any> & AbstractModel, K extends keyof T>(fields: [K, 1 | -1][], config?: {
    unique?: boolean;
    sparse?: boolean;
}) => (constructor: Constructor<T>) => void;
export declare const Column: (props: Partial<ColumnProps>) => (prototype: AbstractModel, propertyKey: string) => void;
export declare const FK: (config: {
    model?: Constructor<AbstractModel>;
    cascade?: CascadeType;
}) => Partial<ColumnProps>;
export {};
