export const _ = (name, code) => (message) => ({ name, code, message });
export const Errors = {
    //-- server errors
    SYSTEM_ERROR: _("SYSTEM_ERROR", 500),
    QUERY_ERROR: _("QUERY_ERROR", 500),
    CANNOT_LOCK: _("CANNOT_LOCK", 500),
    //-- client errors
    VALIDATION_ERROR: _("VALIDATION_ERROR", 400),
    ACCESS_DENIED: _("ACCESS_DENIED", 403),
    TFA_REQUIRED: _("TFA_REQUIRED", 403),
    SESSION_EXPIRED: _("SESSION_EXPIRED", 403),
    NOT_FOUND: _("NOT_FOUND", 404),
    HTTP_REQUEST_ERROR: _("HTTP_REQUEST_ERROR", 405),
    BAD_STATE: _("BAD_STATE", 422),
    TOKEN_EXPIRED: _("TOKEN_EXPIRED", 422),
    INVALID_TOKEN: _("INVALID_TOKEN", 422),
    QUERY_UNIQUE_CONSTRAINT: _("QUERY_UNIQUE_CONSTRAINT", 422),
    QUERY_CASCADE_RESTRICTION: _("QUERY_CASCADE_RESTRICTION", 422),
    INVALID_CREDENTIALS: _("INVALID_CREDENTIALS", 422),
    REQUEST_RATE_LIMIT: _("REQUEST_RATE_LIMIT", 429),
};
export class ErrorBuilder {
    static forConfigNotFound(configName) {
        return Errors.SYSTEM_ERROR(`Config not set for ${configName}`);
    }
    static forTransientUsedWithInitable(constructorFn) {
        return Errors.SYSTEM_ERROR(`Initable cannot be used with transient, found in ${constructorFn.name}`);
    }
    static forInvalidInjection(message) {
        return Errors.SYSTEM_ERROR(message);
    }
    static forInjectionCircularDependency(constructorFns) {
        return Errors.SYSTEM_ERROR(`Circular dependency injection detected between: ${constructorFns.reduce((list, constructor) => `${list}${constructor.name};`, "")}`);
    }
    static forInjectionNotFound(constructorFn) {
        return Errors.SYSTEM_ERROR(`Injection not found for: ${constructorFn.name}`);
    }
    static forMutipleInjectionsFound(constructorFn, candidates) {
        return Errors.SYSTEM_ERROR(`Mutiple injection candidate found for: ${constructorFn.name}, namely: ${candidates.reduce((list, candidate) => `${list}${candidate.name};`, "")}`);
    }
    static forMutipleProvidersFound(constructorFn, candidates) {
        return Errors.SYSTEM_ERROR(`Mutiple injection candidate found for: ${constructorFn.name}, namely: ${candidates.reduce((list, candidate) => `${list}${candidate.name};`, "")}`);
    }
}
